
// ArrayBinaryTree:  Array Implementation of a Binary Tree
// Note that root node is at index 0

public class ArrayBinaryTree implements BinaryTree
{
	
	public static final int CAP = 1000;
	private Object S[];
	private int size;
	
	public ArrayBinaryTree()
	{
		S = new Object[CAP];
		for( int i = 0; i < CAP; i++ )
		{
			S[i] = null;
		}
		size = 0;
	}
	
	public int size()
	{
		return size;
	}
	
	public boolean isEmpty()
	{
		return (size == 0);
	}
	
	public boolean isInternal( Position v )
	{
		return ( hasLeft(v) || hasRight(v) );
	}
	
	public boolean isExternal( Position v )
	{
		return ( !isInternal(v) );
	}
	
	private int getIndex( Position p )
	{
		Index i = (Index) p;
		return i.getIndex();
	}
	
	public boolean isRoot( Position p )
	{
		int v = getIndex( p );
		return (v == 0);
	}
	
	public boolean hasLeft( Position p )
	{
		int v = getIndex( p );
		checkOccupiedPosition( v );
		int leftPos = 2*v + 1;
		if ( leftPos < CAP && S[leftPos] != null )
			return true;
		else
			return false;
	}
	
	public boolean hasRight( Position p )
	{
		int v = getIndex( p );
		checkOccupiedPosition( v );
		int rightPos = 2*v + 2;
		if ( rightPos < CAP && S[rightPos] != null )
			return true;
		else
			return false;
	}
	
	public Position root()
	{
		return new Index( 0 );
	}

	public Position left( Position p )
	{
		int v = getIndex( p );
		checkOccupiedPosition( v );
		checkPosition( 2*v + 1 );
		return new Index( 2*v + 1 );
	}

	public Position right( Position p )
	{
		int v = getIndex( p );
		checkOccupiedPosition( v );
		checkPosition( 2*v + 2 );
		return new Index( 2*v + 2 );
	}

	public Position parent( Position p ) throws NoSuchNodeException
	{
		int v = getIndex( p );
		checkPosition( v );
		if ( isRoot( p ) )
			throw new NoSuchNodeException();
		return new Index( (v-1)/2 );
	}
	
	public Object getElement( Position p )
	{
		int v = getIndex( p );
		checkOccupiedPosition( v );
		return S[v];
	}
	
	public Position sibling( Position p )
	{
		int v = getIndex( p );
		checkOccupiedPosition( v );
		Position par = parent( p );
		Position lc = left( par );
		if ( v == getIndex( lc ) )
			return right( par );
		else
			return lc;
	}
	
	// replaces element at position v, returns previous contents
	public Object replace( Position p, Object o ) throws NoSuchNodeException
	{
		int v = getIndex( p );
		checkPosition( v );
		Object temp = S[v];
		if ( temp == null )
			return new NoSuchNodeException();
		S[v] = o;
		return temp;
	}
	
	public Position addRoot( Object e ) throws NodeExistsException
	{
		if ( S[0] != null )
			throw new NodeExistsException();
		S[0] = e;
		size++;
		return new Index( 0 );
	}

	public Position insertLeft( Position p, Object e ) throws NodeExistsException
	{
		int v = getIndex( p );
		checkOccupiedPosition( v );
		if ( hasLeft( p ) )
			throw new NodeExistsException();
		Position lc = left( p );
		S[getIndex(lc)] = e;
		size++;
		return lc;
	}

	public Position insertRight( Position p, Object e ) throws NodeExistsException
	{
		int v = getIndex( p );
		checkOccupiedPosition( v );
		if ( hasRight( p ) )
			throw new NodeExistsException();
		Position rc = right( p );
		S[getIndex(rc)] = e;
		size++;
		return rc;
	}
	
	// remove a leaf node, return object originally at that node
	public Object remove( Position p ) throws NoSuchNodeException, NotALeafException
	{
		int v = getIndex( p );
		checkPosition( v );
		if ( S[v] == null )
			throw new NoSuchNodeException();
		if ( isInternal( p ) )
			throw new NotALeafException();
		Object temp = S[v];
		S[v] = null;
		size--;
		return temp;
	}

	// convenience method used to check if int parameter is a valid position
	public void checkPosition( int v ) throws InvalidPositionException
	{
		if ( !( v >= 0 && v < CAP ) )
			throw new InvalidPositionException();
	}

	// convenience method used to check if int parameter is a valid position and is occupied
	public void checkOccupiedPosition( int v ) throws InvalidPositionException
	{
		if ( !isOccupied( v ) )
			throw new InvalidPositionException();
	}
	
	public boolean isOccupied( int v )
	{
		return ( v >= 0 && v < CAP  && S[v] != null );
	}

}
