
// LinkedBinaryTree:  Linked Implementation of a Binary Tree
// Requires BTNode class

public class LinkedBinaryTree implements BinaryTree
{
	private BTNode root;
	private int size;
	
	public LinkedBinaryTree()
	{
		root = null;
		size = 0;
	}
	
	public int size()
	{
		return size;
	}
	
	public boolean isEmpty()
	{
		return (size == 0);
	}
	
	public boolean isInternal( Position v )
	{
		return ( hasLeft(v) || hasRight(v) );
	}
	
	public boolean isExternal( Position v )
	{
		return ( !isInternal(v) );
	}
	
	public boolean isRoot( Position p )
	{
		BTNode v = (BTNode) p;
		checkPosition( v );
		return (v == root);
	}
	
	public boolean hasLeft( Position p )
	{
		BTNode v = (BTNode) p;
		checkPosition( v );
		return (v.getLeft() != null);
	}
	
	public boolean hasRight( Position p )
	{
		BTNode v = (BTNode) p;
		checkPosition( v );
		return (v.getRight() != null);
	}
	
	public Position root()
	{
		return root;
	}

	public Position left( Position p )
	{
		BTNode v = (BTNode) p;
		checkPosition( v );
		return v.getLeft();
	}

	public Position right( Position p )
	{
		BTNode v = (BTNode) p;
		checkPosition( v );
		return v.getRight();
	}

	public Position parent( Position p )
	{
		BTNode v = (BTNode) p;
		checkPosition( v );
		return v.getParent();
	}

	public Object getElement( Position p )
	{
		BTNode v = (BTNode) p;
		checkPosition( v );
		return v.getElement();
	}
	
	public Position sibling( Position p )
	{
		BTNode v = (BTNode) p;
		checkPosition( v );
		Position par = parent( v );
		Position lc = left( par );
		if ( v == lc )
			return right(par);
		else
			return lc;
	}
	
	// replaces element at position p, returns previous contents
	public Object replace( Position p, Object o ) throws NoSuchNodeException
	{
		BTNode v = (BTNode) p;
		if ( v == null )
			return new NoSuchNodeException();
		Object temp = v.getElement();
		v.setElement( o );
		return temp;
	}
	
	public Position addRoot( Object e ) throws NodeExistsException
	{
		if ( root != null )
			throw new NodeExistsException();
		root = new BTNode( e );
		size++;
		return root;
	}

	public Position insertLeft( Position p, Object e ) throws NodeExistsException
	{
		BTNode v = (BTNode) p;
		checkPosition( v );
		if ( hasLeft( v ) )
			throw new NodeExistsException();
		BTNode temp = new BTNode( e, null, null, v );
		v.setLeft( temp );
		size++;
		return temp;
	}

	public Position insertRight( Position p, Object e ) throws NodeExistsException
	{
		BTNode v = (BTNode) p;
		checkPosition( v );
		if ( hasRight( v ) )
			throw new NodeExistsException();
		BTNode temp = new BTNode( e, null, null, v );
		v.setRight( temp );
		size++;
		return temp;
	}
	
	// remove a leaf node, return object originally at that node
	public Object remove( Position p ) throws NoSuchNodeException, NotALeafException
	{
		BTNode v = (BTNode) p;
		if ( v == null )
			throw new NoSuchNodeException();
		if ( isInternal( v ) )
			throw new NotALeafException();
		Object temp = v.getElement();
		if (v.getParent().getLeft() == v)
			v.getParent().setLeft( null );
		else
			v.getParent().setRight( null );
		size--;
		return temp;
	}

	// convenience method used to check if BTNode parameter is a valid position
	public void checkPosition( BTNode v ) throws InvalidPositionException
	{
		if ( v == null )
			throw new InvalidPositionException();
	}

}